// ########################################################################
// ## randgraph.cc                                                       ##
// ## Zufllige Erzeugung von Graphen                                    ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#define RandGraphName "RandGraph 1.0"

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>

#include "../tdgf/tdgf.h"
#include "../tdgf/graph.h"
#include "../tdgf/graph.icc"
#include "../zufall/zufall.h"


// Initialisierung des Kantenarrays
// grad[] ist ein Array der Lnge knoten mit den Gradzahlen der Knoten
// In menge[] wird eine Liste von grad[i] Labels des i-ten Knoten fr
// i in aufsteigender Ordnung erstellt.
// Also z.B. 1,1,1,2,3,3 fr Gradzahlen 3 1 2
void Initialisiere(card32 knoten, card32* grad, card32* menge)
{
   card32 i,j,k;

   k = 0;
   for(i = 1;i <= knoten;i++) {
      for(j = 0;j < grad[i];j++) {
         k++;
         menge[k] = i;
      }      
   }
}


// Erzeugung eines Graphen
// knoten enthlt die Anzahl der Knoten
// kanten enthlt die Anzahl der Kanten
// grad[i] ist der Grad des i-ten Knotens
// Nach der Ausfhrung sind die Kanten des Graphen in folgender Weise
//   gespeichert: 
//   (rot[1],rot[2]), ..., (rot[2 * kanten - 1],rot[2 * kanten])
bool RandGraph(card32 knoten, card32 kanten, 
               card32* grad,  card32* rot,
               Graph* graph)
{
   bool   good;
   card32 i,t,x;
   card32 zaehler = 0;

   Initialisiere(knoten,grad,rot);
   good = false;

   while(!good) {
      good = false;
      while(!good) {
         // Zhler berprfen. Falls zu gro -> Graph ist mit angegebenen
         // Daten mit hoher Wahrscheinlichkeit nicht mglich
         zaehler++;
         if(zaehler >= (kanten * 100)) {
            return(false);
         }

         // Neuen Versuch starten, einen Graphen zu erzeugen
         graph->Neu();
         good = true;
         i = kanten;
         while((i >= 1) && good) {
            x = Random(2 * i - 1);
            // Auf Schleifen berprfen
            if(rot[x] == rot[2 * i]) {
               good = false;
            }
            else {
               // Auf multi-edges berprfen
               if(graph->Verbunden(rot[x] - 1,rot[2 * i] - 1)) {
                  good = false;
               }
               else {
                  t = rot[x];
                  rot[x] = rot[2 * i - 1];
                  rot[2 * i - 1] = t;
                  graph->Verbinde(rot[2 * i - 1] - 1,rot[2 * i] - 1);
                  i--;
               }
            }
         }
      }
   }

   // Labels setzen
   char label[16];
   for(i = 1;i <= knoten;i++) {
      sprintf(label,"%d",i);
      graph->SetzeLabel(i - 1,label);
   }

   return(true);
}


// Hauptprogramm
int32 main(card32 argc, char** argv)
{
   card32 knoten        = 0;
   card32 kstart        = 0;
   card32 kende         = 0;
   card32 gstart        = 1;
   card32 gende         = 4;
   card32 anzahl        = 1;
   card32 erfolglos     = 0;
   card32 erfolg        = 0;
   card32 abbruchAnzahl = 0;
   bool bitMatrix   = true;
   char* titel      = NULL;
   char* copyright  = NULL;
   char* programm   = RandGraphName;
   char* name       = NULL;
   bool  ausgabe    = false;
   ofstream           os;
   card32             i,j;

   if(argc == 1) {
      cerr << "Aufruf: " << argv[0] << " {Parameter}" << endl;
      cerr << "-a[#]    - Minimale Knotenzahl setzen" << endl;
      cerr << "-b[#]    - Maximale Knotenzahl setzen" << endl;
      cerr << "-k[#]    - Knotenzahl setzen (entspricht min == max)" << endl;
      cerr << "-g[#]    - Minimale Gradzahl setzen" << endl;
      cerr << "-h[#]    - Maximale Gradzahl setzen" << endl;
      cerr << "-#[#]    - Anzahl der zu erzeugenden Graphen" << endl;
      cerr << "-s[#]    - Maximale Anzahl von Fehlversuchen" << endl;
      cerr << "-f[Name] - Dateiname fr Ausgabe (TDGF)" << endl;
      cerr << "-t[Text] - Titel setzen" << endl;
      cerr << "-c[Text] - Copyright setzen" << endl;
      cerr << "-y       - Adjazenzmatrix im Byte-Format statt Bit-Format speichern" << endl;
      cerr << "-$       - Ausgabe zustzlich auf Bildschirm" << endl;
      exit(1);
   }

   // Parameter auswerten
   for(i = 1;i < argc;i++) {
      if(!(strncmp(argv[i],"-t",2)))      titel     = &argv[i][2];
      else if(!(strncmp(argv[i],"-c",2))) copyright = &argv[i][2];
      else if(!(strncmp(argv[i],"-a",2))) kstart    = atol(&argv[i][2]);
      else if(!(strncmp(argv[i],"-b",2))) kende     = atol(&argv[i][2]);
      else if(!(strncmp(argv[i],"-k",2))) knoten    = atol(&argv[i][2]);
      else if(!(strncmp(argv[i],"-g",2))) gstart    = atol(&argv[i][2]);
      else if(!(strncmp(argv[i],"-h",2))) gende     = atol(&argv[i][2]);
      else if(!(strncmp(argv[i],"-#",2))) anzahl    = atol(&argv[i][2]);
      else if(!(strncmp(argv[i],"-f",2))) name      = &argv[i][2];
      else if(!(strcmp(argv[i],"-y")))    bitMatrix = false;
      else if(!(strcmp(argv[i],"-$")))    ausgabe = true;  
   } 
   if(name == NULL) ausgabe = true;
   if(anzahl == 0) anzahl = 1;
   if(knoten == 0) {
      if(kstart < 1) kstart = 1;
      if(kende < kstart) kende = kstart;
   }
   else {
      if(knoten < 1) knoten = 1;
      kstart = knoten;
      kende  = knoten;
   }
   if(gstart < 1) gstart = 1;
   if(gende < gstart) gende = gstart;
   if(titel == NULL)
      titel = "Zufllig erzeugte Graphen";
   if(name) {
      if(!ErstelleTDGF(os,name,titel,copyright,programm)) {
         cerr << "Kann Datei \"" << name << "\" nicht erstellen!" << endl;
         exit(1);
      }
   }
   if(abbruchAnzahl == 0) abbruchAnzahl = 20 * anzahl;
   if(abbruchAnzahl < 5 * anzahl)  abbruchAnzahl = 5 * anzahl;


   // Zufallszahlengenerator initialisieren
   Randomize();


   // Informationen ausgeben
   cout << "Parameter fr die Generierung:" << endl;
   cout << "Knotenzahlen von " << kstart << " bis " << kende << endl;
   cout << "Gradzahlen von " << gstart << " bis " << gende << endl;
   cout << "Maximal " << abbruchAnzahl << " mgliche Fehlversuche" << endl;
   cout << "Anzahl:    " << anzahl << endl;
   cout << "Titel:     " << titel << endl;
   cout << "Copyright: " << copyright << endl << endl;
   
   j = 0;
   while(j < anzahl) {
      card32  kanten;
      card32* grad = NULL;

      // Zufllige Graph-Daten erzeugen
      do {
         kanten = 0;
         knoten = Random(1 + kende - kstart) + kstart - 1;
         if(grad) delete grad;
         grad = new card32[knoten + 1];
         for(i = 1;i <= knoten;i++) {
            grad[i] = Random(1 + min(gende,knoten) - gstart) + gstart - 1;
            kanten += grad[i];
         }
      } while((kanten % 2) == 1);
      kanten /= 2;

      // Daten ausgeben
      if(ausgabe == true) {
         cout << "Knoten: " << knoten << endl;   
         cout << "Kanten: " << kanten << endl;
         cout << "Grad:   ";
         for(i = 1;i <= knoten;i++) cout << grad[i] << "  ";
         cout << endl;
      }

      // Graph erzeugen
      card32* rot = new card32[2 * kanten + 10];
      for(i = 0;i <= 2 * kanten + 1;i++)
         rot[i] = 0;

      Graph graph(knoten,bitMatrix ? BIT_MATRIX : BYTE_MATRIX);
      if(RandGraph(knoten,kanten,grad,rot,&graph)) {    
         if(ausgabe) {
            cout << "#" << j + 1 << ":" << endl;
            cout << graph << endl;
         }
         if(name) graph.Schreibe(os);
         erfolg++;
         j++;
      }
      else {
         if(ausgabe) {
            cout << "Mit diesen Daten ist die Erzeugung eines Graphen mit hoher" << endl;
            cout << "Wahrscheinlichkeit nicht mglich!" << endl << endl;
         }
         erfolglos++;
         if(erfolglos > abbruchAnzahl) {
            cerr << "Abbruch! Startparameter ergaben zu viele erfolglose Versuche!" << endl;
            j = anzahl;
         }
      }    
      delete grad;
      delete rot;
   }

   cout << "Erzeugte Graphen:    " << erfolg << endl;
   cout << "Erfolglose Versuche: " << erfolglos << endl;
 
   return 0;
}
