// ########################################################################
// ## trigraphconv.cc                                                    ##
// ## Konverter fr TriGraph-Graphen in das TDGF-Format                  ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>

#include "tdgf.h"


// TriGraph-Definitionen
#define MAGIC_ID 0x7466
#define AdjazenzBytes(n) ((n * n + 1) / 2)


// Einen Datensatz vom TriGraph-Format ins TDGF-Format konvertieren
bool Konvertiere(ifstream& is, ofstream& of)
{
   card16 id,knoten,kanten,gradPartition[3];
   char   titel[80];   

   is.read((void *)&id, sizeof(id)); 
   if((is.good()) && (id == MAGIC_ID)) {
      // GRDT-Datensatz schreiben
      is.read((void *)&knoten,sizeof(kanten));
      is.read((void *)&kanten,sizeof(kanten));
      is.read((void *)&gradPartition,sizeof(gradPartition));
      SchreibeGRDT(of,0,knoten,kanten,AdjazenzBytes(knoten));

      // Adjazenzmatrix schreiben
      unsigned char* adjazenzMatrix = new unsigned char[AdjazenzBytes(knoten)];
      is.read((void *)adjazenzMatrix,AdjazenzBytes(knoten));
      of.write((void *)adjazenzMatrix,AdjazenzBytes(knoten));
      delete adjazenzMatrix;

      // GRTT schreiben
      sprintf(titel,"%02d / %02d / %02d",
                    gradPartition[0],gradPartition[1],gradPartition[2]);
      SchreibeGRTT(of,titel);     
      
      if(!(is.fail())) {
         return(true);
      }
   }
   return(false);
}


// Hauptprogramm
int32 main(int32 argc, char** argv)
{
   if(argc < 3) {
      cerr << "Aufruf: " << argv[0] << " [TriGraph-Datei] [TDGF-Ausgabe-Datei] {Titel} {Copyright}" << endl;
      exit(1);
   }
  
   char* titel     = NULL;
   char* copyright = NULL;
   char* programm  = "TriGraphConv 1.0";

   if(argc > 3) titel     = argv[3];
   if(argc > 4) copyright = argv[4];

   ifstream is(argv[1],ios::binary | ios::in);
   if(is.good()) {
      ofstream of;
      if(ErstelleTDGF(of,argv[2],titel,copyright,programm)) {
         while(Konvertiere(is,of));
      }
      else cerr << "Fehler beim ffnen der Ausgabe-Datei!" << endl;
   } else cerr << "Kann Datei nicht ffnen!" << endl;

   return 0;
}
